@@ -38,7 +38,6 @@ from neutron.agent.l3 import l3_agent_extension_api as l3_ext_api
 from neutron.agent.l3 import l3_agent_extensions_manager as l3_ext_manager
 from neutron.agent.l3 import legacy_router
 from neutron.agent.l3 import namespace_manager
-from neutron.agent.l3 import namespaces
 from neutron.agent.l3 import router_processing_queue as queue
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
@@ -58,10 +57,6 @@ from neutron import context as n_context
 from neutron import manager
 
 LOG = logging.getLogger(__name__)
-# TODO(Carl) Following constants retained to increase SNR during refactoring
-NS_PREFIX = namespaces.NS_PREFIX
-INTERNAL_DEV_PREFIX = namespaces.INTERNAL_DEV_PREFIX
-EXTERNAL_DEV_PREFIX = namespaces.EXTERNAL_DEV_PREFIX
 
 # Number of routers to fetch from server at a time on resync.
 # Needed to reduce load on server side and to speed up resync on agent side.
@@ -250,7 +245,7 @@ class L3NATAgent(ha.AgentMixin,
         super(L3NATAgent, self).__init__(host=self.conf.host)
 
         self.target_ex_net_id = None
-        self.use_ipv6 = ipv6_utils.is_enabled()
+        self.use_ipv6 = ipv6_utils.is_enabled_and_bind_by_default()
 
         self.pd = pd.PrefixDelegation(self.context, self.process_monitor,
                                       self.driver,
@@ -313,6 +308,7 @@ class L3NATAgent(ha.AgentMixin,
     def _create_router(self, router_id, router):
         args = []
         kwargs = {
+            'agent': self,
             'router_id': router_id,
             'router': router,
             'use_ipv6': self.use_ipv6,
@@ -321,7 +317,6 @@ class L3NATAgent(ha.AgentMixin,
         }
 
         if router.get('distributed'):
-            kwargs['agent'] = self
             kwargs['host'] = self.host
 
         if router.get('distributed') and router.get('ha'):
@@ -386,7 +381,7 @@ class L3NATAgent(ha.AgentMixin,
         registry.notify(resources.ROUTER, events.BEFORE_DELETE,
                         self, router=ri)
 
-        ri.delete(self)
+        ri.delete()
         del self.router_info[router_id]
 
         registry.notify(resources.ROUTER, events.AFTER_DELETE, self, router=ri)
@@ -461,7 +456,7 @@ class L3NATAgent(ha.AgentMixin,
         self._router_added(router['id'], router)
         ri = self.router_info[router['id']]
         ri.router = router
-        ri.process(self)
+        ri.process()
         registry.notify(resources.ROUTER, events.AFTER_CREATE, self, router=ri)
         self.l3_ext_manager.add_router(self.context, router)
 
@@ -470,7 +465,7 @@ class L3NATAgent(ha.AgentMixin,
         ri.router = router
         registry.notify(resources.ROUTER, events.BEFORE_UPDATE,
                         self, router=ri)
-        ri.process(self)
+        ri.process()
         registry.notify(resources.ROUTER, events.AFTER_UPDATE, self, router=ri)
         self.l3_ext_manager.update_router(self.context, router)
 
@@ -572,10 +567,6 @@ class L3NATAgent(ha.AgentMixin,
         timestamp = timeutils.utcnow()
         router_ids = []
         chunk = []
-        is_snat_agent = (self.conf.agent_mode ==
-                         lib_const.L3_AGENT_MODE_DVR_SNAT)
-        is_dvr_only_agent = (self.conf.agent_mode ==
-                             lib_const.L3_AGENT_MODE_DVR)
         try:
             router_ids = self.plugin_rpc.get_router_ids(context)
             # fetch routers by chunks to reduce the load on server and to
@@ -591,12 +582,14 @@ class L3NATAgent(ha.AgentMixin,
                         # need to keep fip namespaces as well
                         ext_net_id = (r['external_gateway_info'] or {}).get(
                             'network_id')
+                        is_snat_agent = (self.conf.agent_mode ==
+                            lib_const.L3_AGENT_MODE_DVR_SNAT)
                         if ext_net_id:
                             ns_manager.keep_ext_net(ext_net_id)
-                        elif is_snat_agent and not r.get('ha'):
+                        elif is_snat_agent:
                             ns_manager.ensure_snat_cleanup(r['id'])
                     # For HA routers check that DB state matches actual state
-                    if r.get('ha') and not is_dvr_only_agent:
+                    if r.get('ha'):
                         self.check_ha_state_for_router(
                             r['id'], r.get(l3_constants.HA_ROUTER_STATE_KEY))
                     update = queue.RouterUpdate(
