@@ -69,7 +69,8 @@ fake_subnet2 = dhcp.DictModel(dict(id='dddddddd-dddd-dddd-dddddddddddd',
 
 fake_subnet3 = dhcp.DictModel(dict(id='bbbbbbbb-1111-2222-bbbbbbbbbbbb',
                               network_id='12345678-1234-5678-1234567890ab',
-                              cidr='192.168.1.1/24', enable_dhcp=True))
+                              cidr='192.168.1.1/24', enable_dhcp=True,
+                              ip_version=4))
 
 fake_ipv6_subnet = dhcp.DictModel(dict(id='bbbbbbbb-1111-2222-bbbbbbbbbbbb',
                               network_id='12345678-1234-5678-1234567890ab',
@@ -82,7 +83,7 @@ fake_meta_subnet = dhcp.DictModel(dict(id='bbbbbbbb-1111-2222-bbbbbbbbbbbb',
                                   network_id='12345678-1234-5678-1234567890ab',
                                   cidr='169.254.169.252/30',
                                   gateway_ip='169.254.169.253',
-                                  enable_dhcp=True))
+                                  enable_dhcp=True, ip_version=4))
 
 fake_fixed_ip1 = dhcp.DictModel(dict(id='', subnet_id=fake_subnet1.id,
                                 ip_address='172.9.9.9'))
@@ -958,7 +959,7 @@ class TestDhcpAgentEventHandler(base.BaseTestCase):
     def test_subnet_create_restarts_with_dhcp_disabled(self):
         payload = dict(subnet=dhcp.DictModel(
               dict(network_id=fake_network.id, enable_dhcp=False,
-                   cidr='99.99.99.0/24')))
+                   cidr='99.99.99.0/24', ip_version=4)))
         self.cache.get_network_by_id.return_value = fake_network
         new_net = copy.deepcopy(fake_network)
         new_net.subnets.append(payload['subnet'])
@@ -1342,11 +1343,6 @@ class FakePort1(object):
         self.id = 'eeeeeeee-eeee-eeee-eeee-eeeeeeeeeeee'
 
 
-class FakePort2(object):
-    def __init__(self):
-        self.id = 'ffffffff-ffff-ffff-ffff-ffffffffffff'
-
-
 class FakeV4Subnet(object):
     def __init__(self):
         self.id = 'dddddddd-dddd-dddd-dddd-dddddddddddd'
@@ -1356,27 +1352,12 @@ class FakeV4Subnet(object):
         self.enable_dhcp = True
 
 
-class FakeV6Subnet(object):
-    def __init__(self):
-        self.id = 'ffffffff-ffff-ffff-ffff-ffffffffffff'
-        self.ip_version = 6
-        self.cidr = '2001:db8:0:1::/64'
-        self.gateway_ip = '2001:db8:0:1::1'
-        self.enable_dhcp = True
-
-
 class FakeV4SubnetOutsideGateway(FakeV4Subnet):
     def __init__(self):
         super(FakeV4SubnetOutsideGateway, self).__init__()
         self.gateway_ip = '192.168.1.1'
 
 
-class FakeV6SubnetOutsideGateway(FakeV6Subnet):
-    def __init__(self):
-        super(FakeV6SubnetOutsideGateway, self).__init__()
-        self.gateway_ip = '2001:db8:1:1::1'
-
-
 class FakeV4SubnetNoGateway(object):
     def __init__(self):
         self.id = 'eeeeeeee-eeee-eeee-eeee-eeeeeeeeeeee'
@@ -1386,15 +1367,6 @@ class FakeV4SubnetNoGateway(object):
         self.enable_dhcp = True
 
 
-class FakeV6SubnetNoGateway(object):
-    def __init__(self):
-        self.id = 'ffffffff-ffff-ffff-ffff-ffffffffffff'
-        self.ip_version = 6
-        self.cidr = '2001:db8:1:0::/64'
-        self.gateway_ip = None
-        self.enable_dhcp = True
-
-
 class FakeV4Network(object):
     def __init__(self):
         self.id = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
@@ -1403,39 +1375,24 @@ class FakeV4Network(object):
         self.namespace = 'qdhcp-aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
 
 
-class FakeDualNetwork(object):
-    def __init__(self):
-        self.id = 'dddddddd-dddd-dddd-dddd-dddddddddddd'
-        self.subnets = [FakeV4Subnet(), FakeV6Subnet()]
-        self.ports = [FakePort1(), FakePort2()]
-        self.namespace = 'qdhcp-dddddddd-dddd-dddd-dddd-dddddddddddd'
-
-
 class FakeV4NetworkOutsideGateway(FakeV4Network):
     def __init__(self):
         super(FakeV4NetworkOutsideGateway, self).__init__()
         self.subnets = [FakeV4SubnetOutsideGateway()]
 
 
-class FakeDualNetworkOutsideGateway(FakeDualNetwork):
-    def __init__(self):
-        super(FakeDualNetworkOutsideGateway, self).__init__()
-        self.subnets = [FakeV4SubnetOutsideGateway(),
-                        FakeV6SubnetOutsideGateway()]
-
-
-class FakeDualNetworkNoSubnet(object):
+class FakeV4NetworkNoSubnet(object):
     def __init__(self):
         self.id = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
         self.subnets = []
         self.ports = []
 
 
-class FakeDualNetworkNoGateway(object):
+class FakeV4NetworkNoGateway(object):
     def __init__(self):
         self.id = 'cccccccc-cccc-cccc-cccc-cccccccccccc'
-        self.subnets = [FakeV4SubnetNoGateway(), FakeV6SubnetNoGateway()]
-        self.ports = [FakePort1(), FakePort2()]
+        self.subnets = [FakeV4SubnetNoGateway()]
+        self.ports = [FakePort1()]
 
 
 class TestDeviceManager(base.BaseTestCase):
@@ -1790,7 +1747,7 @@ class TestDeviceManager(base.BaseTestCase):
             network = FakeV4Network()
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertFalse(device.route.delete_gateway.called)
         device.route.add_gateway.assert_called_once_with('192.168.0.1')
 
@@ -1804,7 +1761,7 @@ class TestDeviceManager(base.BaseTestCase):
             network = FakeV4NetworkOutsideGateway()
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertFalse(device.route.delete_gateway.called)
         device.route.add_route.assert_called_once_with('192.168.1.1',
                                                        scope='link')
@@ -1816,141 +1773,104 @@ class TestDeviceManager(base.BaseTestCase):
             device = mock.Mock()
             mock_IPDevice.return_value = device
             device.route.get_gateway.return_value = None
-            network = FakeDualNetworkNoSubnet()
+            network = FakeV4NetworkNoSubnet()
             network.namespace = 'qdhcp-1234'
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertFalse(device.route.delete_gateway.called)
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_no_subnet_delete_gateway(self):
         dh = dhcp.DeviceManager(cfg.CONF, None)
-        v4_gateway = '192.168.0.1'
-        v6_gateway = '2001:db8:0:1::1'
-        expected = [mock.call(v4_gateway),
-                    mock.call(v6_gateway)]
         with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
             device = mock.Mock()
             mock_IPDevice.return_value = device
-            device.route.get_gateway.side_effect = [
-                dict(gateway=v4_gateway), dict(gateway=v6_gateway)]
-            network = FakeDualNetworkNoSubnet()
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
+            network = FakeV4NetworkNoSubnet()
             network.namespace = 'qdhcp-1234'
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
-        self.assertEqual(2, device.route.delete_gateway.call_count)
-        device.route.delete_gateway.assert_has_calls(expected)
+        self.assertEqual(1, device.route.get_gateway.call_count)
+        device.route.delete_gateway.assert_called_once_with('192.168.0.1')
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_no_gateway(self):
         dh = dhcp.DeviceManager(cfg.CONF, None)
-        v4_gateway = '192.168.0.1'
-        v6_gateway = '2001:db8:0:1::1'
-        expected = [mock.call(v4_gateway),
-                    mock.call(v6_gateway)]
         with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
             device = mock.Mock()
             mock_IPDevice.return_value = device
-            device.route.get_gateway.side_effect = [
-                dict(gateway=v4_gateway), dict(gateway=v6_gateway)]
-            network = FakeDualNetworkNoGateway()
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
+            network = FakeV4NetworkNoGateway()
             network.namespace = 'qdhcp-1234'
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
-        self.assertEqual(2, device.route.delete_gateway.call_count)
-        device.route.delete_gateway.assert_has_calls(expected)
+        self.assertEqual(1, device.route.get_gateway.call_count)
+        device.route.delete_gateway.assert_called_once_with('192.168.0.1')
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_do_nothing(self):
         dh = dhcp.DeviceManager(cfg.CONF, None)
-        v4_gateway = '192.168.0.1'
-        v6_gateway = '2001:db8:0:1::1'
         with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
             device = mock.Mock()
             mock_IPDevice.return_value = device
-            device.route.get_gateway.side_effect = [
-                dict(gateway=v4_gateway), dict(gateway=v6_gateway)]
-            network = FakeDualNetwork()
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
+            network = FakeV4Network()
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertFalse(device.route.delete_gateway.called)
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_change_gateway(self):
         dh = dhcp.DeviceManager(cfg.CONF, None)
-        v4_gateway = '192.168.0.1'
-        old_v4_gateway = '192.168.0.2'
-        v6_gateway = '2001:db8:0:1::1'
-        old_v6_gateway = '2001:db8:0:1::2'
-        expected = [mock.call(v4_gateway),
-                    mock.call(v6_gateway)]
         with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
             device = mock.Mock()
             mock_IPDevice.return_value = device
-            device.route.get_gateway.side_effect = [
-                dict(gateway=old_v4_gateway), dict(gateway=old_v6_gateway)]
-            network = FakeDualNetwork()
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.2')
+            network = FakeV4Network()
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertFalse(device.route.delete_gateway.called)
-        device.route.add_gateway.assert_has_calls(expected)
+        device.route.add_gateway.assert_called_once_with('192.168.0.1')
 
     def test_set_default_route_change_gateway_outside_subnet(self):
         dh = dhcp.DeviceManager(cfg.CONF, None)
-        v4_gateway = '192.168.1.1'
-        old_v4_gateway = '192.168.2.1'
-        v6_gateway = '2001:db8:1:1::1'
-        old_v6_gateway = '2001:db8:2:0::1'
-        add_route_expected = [mock.call(v4_gateway, scope='link'),
-                              mock.call(v6_gateway, scope='link')]
-        add_gw_expected = [mock.call(v4_gateway),
-                           mock.call(v6_gateway)]
         with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
             device = mock.Mock()
             mock_IPDevice.return_value = device
-            device.route.list_onlink_routes.side_effect = [
-                [{'cidr': old_v4_gateway}], []]
-            device.route.get_gateway.side_effect = [
-                dict(gateway=old_v4_gateway), dict(gateway=old_v6_gateway)]
-            network = FakeDualNetworkOutsideGateway()
+            device.route.list_onlink_routes.return_value = (
+                [{'cidr': '192.168.2.1'}])
+            device.route.get_gateway.return_value = dict(gateway='192.168.2.1')
+            network = FakeV4NetworkOutsideGateway()
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertEqual(2, device.route.list_onlink_routes.call_count)
         self.assertFalse(device.route.delete_gateway.called)
-        device.route.delete_route.assert_called_once_with(old_v4_gateway,
+        device.route.delete_route.assert_called_once_with('192.168.2.1',
                                                        scope='link')
-        device.route.add_route.assert_has_calls(add_route_expected)
-        device.route.add_gateway.assert_has_calls(add_gw_expected)
+        device.route.add_route.assert_called_once_with('192.168.1.1',
+                                                       scope='link')
+        device.route.add_gateway.assert_called_once_with('192.168.1.1')
 
     def test_set_default_route_two_subnets(self):
         # Try two subnets. Should set gateway from the first.
         dh = dhcp.DeviceManager(cfg.CONF, None)
-        v4_gateway = '192.168.1.1'
-        v6_gateway = '2001:db8:1:1::1'
-        expected = [mock.call(v4_gateway),
-                    mock.call(v6_gateway)]
         with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
             device = mock.Mock()
             mock_IPDevice.return_value = device
             device.route.get_gateway.return_value = None
-            network = FakeDualNetwork()
+            network = FakeV4Network()
             subnet2 = FakeV4Subnet()
-            subnet2.gateway_ip = v4_gateway
-            subnet3 = FakeV6Subnet()
-            subnet3.gateway_ip = v6_gateway
-            network.subnets = [subnet2, FakeV4Subnet(),
-                               subnet3, FakeV6Subnet()]
+            subnet2.gateway_ip = '192.168.1.1'
+            network.subnets = [subnet2, FakeV4Subnet()]
             dh._set_default_route(network, 'tap-name')
 
-        self.assertEqual(2, device.route.get_gateway.call_count)
+        self.assertEqual(1, device.route.get_gateway.call_count)
         self.assertFalse(device.route.delete_gateway.called)
-        device.route.add_gateway.assert_has_calls(expected)
+        device.route.add_gateway.assert_called_once_with('192.168.1.1')
 
 
 class TestDictModel(base.BaseTestCase):
