@@ -22,7 +22,6 @@ from oslo_utils import uuidutils
 from neutron.agent.common import config as agent_config
 from neutron.agent.l3 import agent as l3_agent
 from neutron.agent.l3 import dvr_local_router as dvr_router
-from neutron.agent.l3 import ha
 from neutron.agent.l3 import link_local_allocator as lla
 from neutron.agent.l3 import router_info
 from neutron.agent.linux import external_process
@@ -31,6 +30,7 @@ from neutron.agent.linux import ip_lib
 from neutron.common import constants as n_const
 from neutron.common import utils as common_utils
 from neutron.conf.agent.l3 import config as l3_config
+from neutron.conf.agent.l3 import ha as ha_conf
 from neutron.conf import common as base_config
 from neutron.extensions import portbindings
 from neutron.tests import base
@@ -51,7 +51,7 @@ class TestDvrRouterOperations(base.BaseTestCase):
         log.register_options(self.conf)
         self.conf.register_opts(agent_config.AGENT_STATE_OPTS, 'AGENT')
         l3_config.register_l3_agent_config_opts(l3_config.OPTS, self.conf)
-        self.conf.register_opts(ha.OPTS)
+        ha_conf.register_l3_agent_ha_opts(self.conf)
         agent_config.register_interface_driver_opts_helper(self.conf)
         agent_config.register_process_monitor_opts(self.conf)
         self.conf.register_opts(interface.OPTS)
@@ -65,7 +65,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
             'neutron.agent.linux.ip_lib.device_exists')
         self.device_exists = self.device_exists_p.start()
 
-        self.ensure_dir = mock.patch('neutron.common.utils.ensure_dir').start()
+        self.ensure_dir = mock.patch(
+            'oslo_utils.fileutils.ensure_tree').start()
 
         mock.patch('neutron.agent.linux.keepalived.KeepalivedManager'
                    '.get_full_config_file_path').start()
@@ -75,7 +76,7 @@ class TestDvrRouterOperations(base.BaseTestCase):
         self.utils_exec = self.utils_exec_p.start()
 
         self.utils_replace_file_p = mock.patch(
-            'neutron.common.utils.replace_file')
+            'neutron_lib.utils.file.replace_file')
         self.utils_replace_file = self.utils_replace_file_p.start()
 
         self.external_process_p = mock.patch(
@@ -151,13 +152,17 @@ class TestDvrRouterOperations(base.BaseTestCase):
         self.router_id = _uuid()
         if not router:
             router = mock.MagicMock()
-        return dvr_router.DvrLocalRouter(agent,
-                                    HOSTNAME,
-                                    self.router_id,
-                                    router,
-                                    self.conf,
-                                    mock.Mock(),
-                                    **kwargs)
+        kwargs['agent'] = agent
+        kwargs['router_id'] = self.router_id
+        kwargs['router'] = router
+        kwargs['agent_conf'] = self.conf
+        kwargs['interface_driver'] = mock.Mock()
+        return dvr_router.DvrLocalRouter(HOSTNAME, **kwargs)
+
+    def _set_ri_kwargs(self, agent, router_id, router):
+        self.ri_kwargs['agent'] = agent
+        self.ri_kwargs['router_id'] = router_id
+        self.ri_kwargs['router'] = router
 
     def test_gw_ns_name(self):
         ri = self._create_router()
@@ -376,8 +381,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
         agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
         router = l3_test_common.prepare_router_data(num_internal_ports=2)
         router['distributed'] = True
-        ri = dvr_router.DvrLocalRouter(
-            agent, HOSTNAME, router['id'], router, **self.ri_kwargs)
+        self._set_ri_kwargs(agent, router['id'], router)
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         ports = ri.router.get(lib_constants.INTERFACE_KEY, [])
         subnet_id = l3_test_common.get_subnet_id(ports[0])
         test_ports = [{'mac_address': '00:11:22:33:44:55',
@@ -433,12 +438,10 @@ class TestDvrRouterOperations(base.BaseTestCase):
         agent.add_arp_entry(None, payload)
 
     def test__update_arp_entry_with_no_subnet(self):
-        ri = dvr_router.DvrLocalRouter(
-            mock.sentinel.agent,
-            HOSTNAME,
-            'foo_router_id',
-            {'distributed': True, 'gw_port_host': HOSTNAME},
-            **self.ri_kwargs)
+        self._set_ri_kwargs(mock.sentinel.agent,
+                            'foo_router_id',
+                            {'distributed': True, 'gw_port_host': HOSTNAME})
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         with mock.patch.object(l3_agent.ip_lib, 'IPDevice') as f:
             ri._update_arp_entry(mock.ANY, mock.ANY, 'foo_subnet_id', 'add')
         self.assertFalse(f.call_count)
@@ -447,8 +450,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
         agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
         router = l3_test_common.prepare_router_data(num_internal_ports=2)
         router['distributed'] = True
-        ri = dvr_router.DvrLocalRouter(
-            agent, HOSTNAME, router['id'], router, **self.ri_kwargs)
+        self._set_ri_kwargs(agent, router['id'], router)
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         subnet_id = l3_test_common.get_subnet_id(
             ri.router[lib_constants.INTERFACE_KEY][0])
         return ri, subnet_id
@@ -526,8 +529,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
         router[n_const.FLOATINGIP_AGENT_INTF_KEY] = agent_gateway_port
         router['distributed'] = True
         agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
-        ri = dvr_router.DvrLocalRouter(
-            agent, HOSTNAME, router['id'], router, **self.ri_kwargs)
+        self._set_ri_kwargs(agent, router['id'], router)
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         self.assertEqual(
             agent_gateway_port[0],
             ri.get_floating_agent_gw_interface(fake_network_id))
@@ -551,11 +554,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
         router[lib_constants.FLOATINGIP_KEY] = fake_floatingips['floatingips']
         router['distributed'] = True
         agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
-        ri = dvr_router.DvrLocalRouter(agent,
-                                  HOSTNAME,
-                                  router['id'],
-                                  router,
-                                  **self.ri_kwargs)
+        self._set_ri_kwargs(agent, router['id'], router)
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         ri.iptables_manager.ipv4['nat'] = mock.MagicMock()
         ri.dist_fip_count = 0
         fip_ns = agent.get_fip_ns(mock.sentinel.ext_net_id)
@@ -575,11 +575,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
                                             agent_mode, expected_call_count):
         router = l3_test_common.prepare_router_data(num_internal_ports=2)
         agent = l3_agent.L3NATAgent(HOSTNAME, self.conf)
-        ri = dvr_router.DvrLocalRouter(agent,
-                                       HOSTNAME,
-                                       router['id'],
-                                       router,
-                                       **self.ri_kwargs)
+        self._set_ri_kwargs(agent, router['id'], router)
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
 
         interface_name, ex_gw_port = l3_test_common.prepare_ext_gw_test(self,
                                                                         ri)
@@ -611,8 +608,8 @@ class TestDvrRouterOperations(base.BaseTestCase):
         self.mock_driver.unplug.reset_mock()
 
         external_net_id = router['gw_port']['network_id']
-        ri = dvr_router.DvrLocalRouter(
-            agent, HOSTNAME, router['id'], router, **self.ri_kwargs)
+        self._set_ri_kwargs(agent, router['id'], router)
+        ri = dvr_router.DvrLocalRouter(HOSTNAME, **self.ri_kwargs)
         ri.remove_floating_ip = mock.Mock()
         agent._fetch_external_net_id = mock.Mock(return_value=external_net_id)
         ri.ex_gw_port = ri.router['gw_port']
